/**
 * (c) Atos IT Solutions and Services GmbH, 2011.
 * Licensed under EUPL 1.1
 */
package com.sag.osami.contextstore.osgi;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.util.tracker.ServiceTrackerCustomizer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sag.osami.api.context.IContextHandler;
import com.sag.osami.contextstore.ContextStore;
import com.sag.osami.contextstore.serialization.ISerializedContextHandler;
import com.sag.osami.contextstore.serialization.ISerializedContextProducer;

/**
 * This class tracks all services which are interested in getting access to the ContextStore, i.e.:
 * 
 * <ul>
 * <li>{@link IContextHandler}s</li>
 * <li>{@link ISerializedContextHandler}s</li>
 * <li>{@link ISerializedContextProducer}s</li>
 * </ul>
 * 
 * <p>
 * It grants access to the ContextStore by calling the <code>setContextStore</code> method of the interested service. If
 * the interested service is a handler, this class also registers it as a handler in the ContextStore.
 */
public class OsgiContextStoreSetter implements ServiceTrackerCustomizer {

	private ContextStore contextStore;

	private Logger log;
	private BundleContext context;

	public OsgiContextStoreSetter(ContextStore contextStore) {
		this.context = OsgiContextStoreActivator.getContext();
		this.contextStore = contextStore;
		this.log = LoggerFactory.getLogger(getClass());
	}

	public Object addingService(ServiceReference reference) {
		Object service = context.getService(reference);
		log.debug("new {} registered", service.getClass().getSimpleName());

		if (service instanceof IContextHandler) {
			IContextHandler handler = (IContextHandler) service;
			contextStore.registerHandler(handler, handler.getDefaultContextTypes());			
		}

		if (service instanceof ISerializedContextProducer)
			((ISerializedContextProducer) service).setContextStore(this.contextStore);

		return service;
	}

	public void removedService(ServiceReference reference, Object service) {
		if (service instanceof IContextHandler)
			contextStore.deregisterHandler((IContextHandler) service);
		log.debug("{} deregistered", service.getClass().getSimpleName());
		context.ungetService(reference);
	}

	public void modifiedService(ServiceReference reference, Object service) {
		// doesn't matter
	}
}
