/**
 * (c) Atos IT Solutions and Services GmbH, 2011.
 * Licensed under EUPL 1.1
 */
package com.sag.osami.contextstore.storage;

import java.util.List;

import com.sag.osami.api.context.ContextEvent;
import com.sag.osami.api.context.IContextStorage;
import com.sag.osami.contextstore.ContextStore;
import com.sag.osami.contextstore.serialization.ISerializationHandler;

/**
 * This interface is used by {@link ContextStore} for all storage-related logic. Any actual ContextStore storage back-end has to implement this interface.
 * <p>
 * The methods are basically the same as in {@link IContextStorage}, but include (de)serialization flags. Payload serialization obviously plays a role in
 * storage-related program logic. On the other hand, serialization details should not be exposed to the regular ContextStore user (i.e. should not be part of
 * the public API).
 * <p>
 * However, it wouldn't suffice to encapsulate and hide serialization completely within the storage back-end by simply using IContextStorage as the internal
 * interface (which does not expose any serialization flags etc.) in the ContextStore: in internal ContextStore logic, on different occasions it is
 * necessary/convenient to obtain IContextEvents with either already deserialized or still serialized payload data; similarly, on different occasions it is
 * necessary/convenient to store IContextEvents with either already serialized or still deserialized payload data.
 * <p>
 * Hence, a storage back-end has to account for the two possible forms of payload data and handle it accordingly. For this, it must have access to the
 * {@link ISerializationHandler}, which is granted by {@link #setSerializer(ISerializationHandler)}.
 * <p>
 * As the ContextStore is responsible for assigning unique IDs to context events, it must be able to retrieve the last given ID during its initialization --
 * this capability also is to be provided by the storage back-end.
 * 
 * @see SerializedContextStorage
 */
public interface ISerializedContextStorage extends IContextStorage {

	/**
	 * Loads a context event with the given ID.
	 * 
	 * @param deserialize
	 *            whether the payload data should remain serialized or should be deserialized
	 * @param contextId
	 *            ID of the context event
	 * @return loaded context event (or null, if there is no context event with given ID)
	 */
	ContextEvent load(boolean deserialize, Long contextId);

	/**
	 * Loads several context events with the given IDs. The order within the result is unspecified.
	 * 
	 * @param deserialize
	 *            whether the payload data should remain serialized or should be deserialized
	 * @param contextIds
	 *            IDs of the context events
	 * @return loaded context events (or a zero-length array, if there are no context events with given IDs)
	 */
	ContextEvent[] load(boolean deserialize, List<Long> contextIds);

	/**
	 * Lazy variant of {@link #load(boolean, List)}, i.e. it loads one ContextEvent at a time during iteration. This operation is optional.
	 * 
	 * @throws UnsupportedOperationException
	 *             if the underlying storage implementation does not support lazy querying
	 * @see #load(boolean, List)
	 */
	Iterable<ContextEvent> loadLazy(boolean deserialize, List<Long> contextIds) throws UnsupportedOperationException;

	/**
	 * Queries several context events.
	 * 
	 * @param deserialize
	 *            whether the payload data should remain serialized or should be deserialized
	 * @param query
	 *            its language/format depends on the actual storage management class.
	 * @param params
	 *            Should be used for parameter binding à la JDBC:
	 *            {@code query("SELECT * FROM CONTEXTSTORE WHERE TYPE = ? AND TIMESTAMP >= ?", "someType", someDate.getTime());}
	 * @return queried context events (or a zero-length array, if there are no according context events)
	 */
	ContextEvent[] query(boolean deserialize, String query, Object... params);

	/**
	 * Lazy variant of {@link #query(boolean, String, Object...)}, i.e. it loads one ContextEvent at a time during iteration. This operation is optional.
	 * 
	 * @throws UnsupportedOperationException
	 *             if the underlying storage implementation does not support lazy querying
	 * @see #query(boolean, String, Object...)
	 */
	Iterable<ContextEvent> queryLazy(boolean deserialize, String query, Object... params) throws UnsupportedOperationException;

	/**
	 * Stores a context event object persistently.
	 * 
	 * @param serialize
	 *            whether the payload data should be serialized or not (because it is already serialized)
	 * @param context
	 *            context event object to be stored
	 */
	void store(boolean serialize, ContextEvent context);

	/**
	 * Grants access to the payload data serialization handler.
	 * 
	 * @param serializer
	 *            payload data serialization handler
	 */
	void setSerializer(ISerializationHandler serializer);

	/**
	 * ContextStore uses the result of this method as the offset for its auto-incremented context event IDs. This method is used only during the initialization
	 * of the ContextStore, it doesn't have to return the current last given ID during the whole lifetime.
	 * 
	 * @return the so-far highest already assigned ID of a persisted context event
	 */
	Long retrieveLastID();
}
